package com.guams.review.controller;

import com.guams.review.exception.AlreadyExistsException;
import com.guams.review.exception.InvalidNameOrPasswordException;
import com.guams.review.exception.NotFoundException;
import com.guams.review.model.AuthorRepository;
import com.guams.review.model.dao.Author;
import com.guams.review.model.dao.Post;
import com.guams.review.service.AuthorService;
import com.guams.review.service.mapper.Mapper;
import com.guams.review.service.mapper.ReturnableAuthor;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.util.Assert;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.List;
import java.util.UUID;

@RestController
@RequestMapping(path = "api/authors")
@RequiredArgsConstructor
public class AuthorController {


    private final PasswordEncoder passwordEncoder;
    private final AuthenticationManager authenticationManager;
    private final AuthorService authorService;
    private final AuthorRepository authorRepository;
    private final Mapper mapper;

    @GetMapping
    public List<ReturnableAuthor> getUsers() {
        return authorService.list();
    }

    @GetMapping("/{id}")
    public ReturnableAuthor findUser(@PathVariable UUID id) {
        Author author = authorService.findById(id).orElseThrow(() -> new NotFoundException("Author not found"));
        return mapper.mapAuthor(author);
    }

    @PutMapping("/{id}")
    public void updateUser(@PathVariable UUID id, @RequestBody Author updatedAuthor) {
        Author authorToUpdate = authorService.findById(id).orElseThrow(() -> new NotFoundException("Author not found"));
        authorService.insert(updatedAuthor.setId(authorToUpdate.getId()));
    }

    @PutMapping(value = "{id}/avatar", consumes = {MediaType.MULTIPART_FORM_DATA_VALUE})
    public void updateUserAvatar(@PathVariable UUID id, @RequestPart MultipartFile avatar) throws IOException {
        Author authorToUpdate = authorService.findById(id).orElseThrow(() -> new NotFoundException("Author not found"));
        authorService.insert(authorToUpdate.setProfilePicture(avatar.getBytes()));
    }

    @DeleteMapping("/{id}")
    public void deleteUser(@PathVariable UUID id) {
        Author authorToDelete = authorService.findById(id).orElseThrow(() -> new NotFoundException("Author not found"));
        authorService.delete(authorToDelete);
    }

    @PutMapping("/{id}/posts")
    public void updateUserPosts(@PathVariable("id") UUID authorId, @RequestBody List<Long> postIds) {
        Author author = authorService.findById(authorId).orElseThrow(() -> new NotFoundException("Author not found"));
        authorService.insertPublications(author.getId(), postIds);
    }

    @GetMapping("/{id}/posts")
    public List<Post> getUserPosts(@PathVariable UUID id) {
        Author author = authorService.findById(id).orElseThrow(() -> new NotFoundException("Author not found"));
        return authorService.listPublicationOfAuthor(author.getId());
    }

    @PostMapping("/login")
    public void authorLogin(@RequestBody Author author) {
        try {
            authenticationManager.authenticate(new UsernamePasswordAuthenticationToken(author.getName(), author.getPassword()));
        }
        catch (Exception e) {
            throw new InvalidNameOrPasswordException(e.getMessage());
        }
    }

    @PostMapping("/register")
    public ResponseEntity<Author> authorRegister(@RequestBody Author author) {
        Assert.isNull(author.getId(), "Author id must be null");
        if (authorRepository.findByName(author.getName()) != null) {
            throw new AlreadyExistsException("Author already exists");
        }
        author.setPassword(passwordEncoder.encode(author.getPassword()));
        return new ResponseEntity<>(authorRepository.save(author).setPassword(""), HttpStatus.CREATED);
    }
}
